# *************************************************************************
#  Licensed Materials - Property of IBM
#  5724-M23
#  Copyright IBM Corporation 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************

from java.lang import System, String
from java.util import Properties
from java.io import FileInputStream, FileOutputStream

import os
import sys
import re
import shutil


def regexp(pattern, string, flags=0):
    if(re.compile(pattern, flags).search(string)==None): return 0
    else: return 1
#endDef

#*******************************************************************************************************
# Procedure to read the scripting client's log file (usually, wsadmin.traceout)
#*******************************************************************************************************
def adminLog():	
    adminlog = System.getProperty("com.ibm.ws.scripting.traceFile")
    print "com.ibm.ws.scripting.traceFile is " + adminlog          
    if ( not os.path.exists( adminlog ) ):
		# Sometimes, there is a trailing blank for this property in wsadmin.properties, strip it off
        adminlog = adminlog[:-1]

    if ( not os.path.exists( adminlog ) ):
        adminlog = ""
    else:
    	print "opening trace file at " + adminlog
        tmp = open( adminlog, "r" )
        adminlog = tmp.read()
        tmp.close()
        
    return adminlog
#endDef 

#*******************************************************************************************************
# Procedure to query the connection status of the admin client ('CONNECTED', 'DISCONNECTED', or 'ERROR')
#*******************************************************************************************************
def getConnectionStatus():
    global connectionStatus
    
    print "trying to get the connection status from wsadmin.traceout file under this profile's logs directory"
    print ""
    adminlog = adminLog()
    print ""
    
    # there is no message in wsadmin.traceout indicating a successful connection
    connectionStatus = "UNKNOWN"
        
    if (regexp("WASX7326I:", adminlog) ):
        # WASX7326I: Loaded properties file "/D:/WebSphere/AppServer51/properties/wsadmin.properties"
        print "message WASX7326I found in the trace file"
        connectionStatus = "CONNECTED"
    if (regexp("WASX7357I:", adminlog) ):
        # WASX7357I: By request, this scripting client is not connected to any server process. Certain configuration and application operations will be available in local mode.
        print "message WASX7326I found in the trace file"
        connectionStatus = "DISCONNECTED"
	if (regexp("WASX7198W:", adminlog) ):
		# "WASX7198W: The configuration service is not running. Configuration commands will not run."
		print "message WASX7326I found in the trace file"
		connectionStatus = "ERROR"
    if (regexp("WASX7246E:", adminlog) ):
        # "WASX7246E: Cannot establish "<conntype>" connection to host "<hostname>" because of an authentication failure. ..."
        print "message WASX7326I found in the trace file"
        connectionStatus = "ERROR"
    if (regexp("WASX7023E:", adminlog) ):
        # "WASX7023E: Error creating "<conntype>" connection to host "<hostname>"; exception information: com.ibm.websphere.management.exception.ConnectorNotAvailableException"
        print "message WASX7326I found in the trace file"
        connectionStatus = "ERROR"
    
    print ""
    print "connection status is ", connectionStatus
#endDef

def getCellType():
	print ""
	cellType = ""
	
	try:
		print "getting cellID"
		cellID = AdminConfig.getid("/Cell:" + System.getProperty("local.cell"));
		print "cellID : ", cellID 
		
		print "getting cell type"
		cellType = AdminConfig.showAttribute(cellID, 'cellType');
		
	except:
		print "Unable to get cell type - exception: ", sys.exc_type, sys.exc_value		
	
	print "cellType : ", cellType
	print ""
	return cellType
#endDef

def isDMgrNode():
	print ""
	isDMgrNode = "False"
	
	try:
		nodeNameParam = '[-nodeName ' + System.getProperty("local.node") + ']'
		
		print "The value of nodeNameParam is ", nodeNameParam
		servers = AdminTask.listServers(nodeNameParam).split(System.getProperty("line.separator"));
		print "The value of servers is ", servers
		
		for server in servers:
			servertype = AdminConfig.showAttribute(server, "serverType");
			print "The value of servertype is ", servertype 
			if (servertype == "DEPLOYMENT_MANAGER"):
				isDMgrNode = "True"
				break;
					
	except:	
		print "Cannot find the node type. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value
		
	print "The value of isDMgrNode is ", isDMgrNode	
	print ""
	return isDMgrNode				
#endDef

def copyFile(source, destination):
	global numerrors;
	print ""
	
	try:						
		print "copy from " + source + " to " + destination							
		shutil.copy( source, destination);
		print "copied from " + source + " to " + destination
			
	except:
		print 'unable to copy file: ', sys.exc_type, sys.exc_value
		numerrors = numerrors + 1;
		
	print ""				
#endDef

def copyDirectory(source, destination):
	global numerrors;
	print ""
	
	try:						
		print "copy from " + source + " to " + destination							
		shutil.copytree( source, destination);
		print "copied from " + source + " to " + destination
			
	except:
		print 'unable to copy file: ', sys.exc_type, sys.exc_value
		numerrors = numerrors + 1;
		
	print ""				
#endDef


def copyRegistrationFiles():
	print ""
	
	profileBSpaceDir = System.getProperty("user.install.root") + "/BusinessSpace";				
	profileBSpaceRegDataDir = profileBSpaceDir + "/registryData";
	
	print "copy widget registration files to ", profileBSpaceRegDataDir
	
	copyFile(System.getProperty("was.install.root") + "/BusinessSpace/BusinessLeader/registryData/busLeaderEndpoints.xml", profileBSpaceRegDataDir + "/busLeaderEndpoints.xml");
	copyFile(System.getProperty("was.install.root") + "/BusinessSpace/BusinessLeader/registryData/busLeaderWidgets.xml", profileBSpaceRegDataDir + "/busLeaderWidgets.xml");

	print ""				
#endDef

def copyTemplateFiles():
	print ""
	
	profileBSpaceDir = System.getProperty("user.install.root") + "/BusinessSpace";				
	profileBSpaceTemplateDataDir = profileBSpaceDir + "/templateData";	
	
	print "copy tempalte files to ", profileBSpaceTemplateDataDir
	
	copyDirectory(System.getProperty("was.install.root") + "/BusinessSpace/BusinessLeader/templateData", profileBSpaceTemplateDataDir);

	print ""				
#endDef

def isPublishingServerInstalled():
	global pubserverDeploymentTargetsList;

	print ""
	isInstalled = "false";
	
	pubServerDeploymentTargetsList = [];
	
	try:
		print "check if the Publishing Server has been configured on this cell or not"
		print ""
		
		installedApps = AdminConfig.list("ApplicationDeployment");
		# @todo string to list needs a revisit to properly check for empty list
		installedApps = installedApps.split(lineSeparator);
		print "installed applications : ", installedApps
		
		for app in installedApps:
			print ""		
			print "application : ", app
			
			modules = AdminConfig.list("WebModuleDeployment", app);
			stringValue = String(modules);
			
			modulesList = modules.split(lineSeparator); 
			print "modules list: ", modulesList
			
			if (stringValue.length() != 0):
							
				for module in modulesList:
					print "module : ", module
					uri = AdminConfig.showAttribute(module, "uri");
					print "uri : ", uri
					
					if uri != "WBMPubServerWeb.war":
						continue;
					
					isInstalled = "true";
					print ""		
					print "is the Publishing Server Application installed : ", isInstalled
										
					deploymentTargets = AdminConfig.list("DeploymentTargetMapping", module);
					stringValue = String(deploymentTargets);
										
					if (stringValue.length() != 0):
						pubserverDeploymentTargetsList = deploymentTargets.split(lineSeparator);
						 
					print ""
					print "deployment targets on which the Publishing Server has been configured : " 
					print ""
											
					for deploymentTarget in pubserverDeploymentTargetsList:
						print deploymentTarget
					break;	
					
					print ""				
					
			if isInstalled != "false":
				break;
						
	except:
		print "Unable to find whether the Publishing Server has been configured in this cell or not - exception: ", sys.exc_type, sys.exc_value

	print "is Publishing Server configured : ", isInstalled	
	print ""	
	return isInstalled;
#endDef

def isBusinessSpaceInstalled():
	global bspaceDeploymentTargetsList;

	print ""
	isInstalled = "false";
	
	bspaceDeploymentTargetsList = [];
	
	try:
		print "check if the Business Space has been configured on this cell or not"
		print ""
		
		installedApps = AdminConfig.list("ApplicationDeployment");
		# @todo string to list needs a revisit to properly check for empty list
		installedApps = installedApps.split(lineSeparator);
		print "installed applications : ", installedApps
		
		for app in installedApps:
			print ""		
			print "application : ", app
			
			modules = AdminConfig.list("WebModuleDeployment", app);
			stringValue = String(modules);
			
			modulesList = modules.split(lineSeparator); 
			#print "modules list: ", modulesList
			
			if (stringValue.length() != 0):
							
				for module in modulesList:
					print "module : ", module
					uri = AdminConfig.showAttribute(module, "uri");
					print "uri : ", uri
					
					if uri != "BSpaceManager.war":
						continue;
					
					isInstalled = "true";
					print ""		
					print "is the Business Space Application installed : ", isInstalled
										
					deploymentTargets = AdminConfig.list("DeploymentTargetMapping", module);
					stringValue = String(deploymentTargets);
										
					if (stringValue.length() != 0):
						bspaceDeploymentTargetsList = deploymentTargets.split(lineSeparator);
						 
					print ""
					print "deployment targets on which the Business Space has been configured : " 
					print ""
											
					for deploymentTarget in bspaceDeploymentTargetsList:
						print deploymentTarget
					break;	
					
					print ""				
					
			if isInstalled != "false":
				break;
						
	except:
		print "Unable to find whether the Business Space has been configured in this cell or not - exception: ", sys.exc_type, sys.exc_value

	print "is Business Space configured : ", isInstalled	
	print ""	
	return isInstalled;
#endDef

def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
    exist = 0
    applications = wsadminToList(AdminApp.list())
    for application in applications:
        if (application == applicationName):
            exist = 1
            return exist
    return exist

def installBusinessLeaderApps(): 
	whetherApplicationExist = checkApplicationExist("BusinessLeaderApp")
	if(whetherApplicationExist == 0):
		installBusinessLeaderServerApp();
	else:
		print "BusinessLeaderServerApp is already installed."	
		
	whetherApplicationExist = checkApplicationExist("BusinessLeaderWidgetsApp")
	if(whetherApplicationExist == 0):
		installBusinessLeaderWidgetsApp();
	else:
		print "BusinessLeaderWidgetsApp is already installed."	  
#endDef		  

def installBusinessLeaderServerApp():
	global pubserverDeploymentTargetsList;
	
	print ""
	print "installing Business Leader Server App"
	
	installableApp = System.getProperty("was.install.root") + "/BusinessSpace/BusinessLeader/" +  "BusinessLeaderApp.ear";		
	print "installing application from : ", installableApp
	
	options = [];
	options.append("-MapModulesToServers");
	
	mapWebModuleOption = [];
	options.append(mapWebModuleOption);
	
	mapWebModuleOptionContent = [];
	mapWebModuleOption.append(mapWebModuleOptionContent);
	
	mapWebModuleOptionContent.append(".*");
	mapWebModuleOptionContent.append(".*");
	
	targetMappings = "";		
	
	for targetMapping in pubserverDeploymentTargetsList:			
		print ""
		
		target = AdminConfig.showAttribute(targetMapping, "target")
		print "target : ", target
		dtName = AdminConfig.showAttribute(target, "name");
		print "target name : ", dtName
		
		try:
			nodeName = AdminConfig.showAttribute(target, "nodeName");
			print "target node name : ", nodeName
			targetMappings  = targetMappings  + "+WebSphere:node="+ nodeName +",server=" + dtName;
			
		except:
			targetMappings  = targetMappings  + "+WebSphere:cluster="+ dtName;
			
	mapWebModuleOptionContent.append(targetMappings);
	
	print ""
	print "options : ", options
	
	AdminApp.install(installableApp, options);
	AdminConfig.save();
	print "installed application from ", installableApp
		
	print ""
#endDef

def installBusinessLeaderWidgetsApp():
	global pubserverDeploymentTargetsList;
	
	print ""
	print "installing Business Leader Widgets App"
	
	installableApp = System.getProperty("was.install.root") + "/BusinessSpace/BusinessLeader/" +  "BusinessLeaderWidgetsApp.ear";		
	print "installing application from : ", installableApp
	
	options = [];
	options.append("-MapModulesToServers");
	
	mapWebModuleOption = [];
	options.append(mapWebModuleOption);
	
	mapWebModuleOptionContent = [];
	mapWebModuleOption.append(mapWebModuleOptionContent);
	
	mapWebModuleOptionContent.append(".*");
	mapWebModuleOptionContent.append(".*");
	
	targetMappings = "";		
	
	for targetMapping in pubserverDeploymentTargetsList:			
		print ""
		
		target = AdminConfig.showAttribute(targetMapping, "target")
		print "target : ", target
		dtName = AdminConfig.showAttribute(target, "name");
		print "target name : ", dtName
		
		try:
			nodeName = AdminConfig.showAttribute(target, "nodeName");
			print "target node name : ", nodeName
			targetMappings  = targetMappings  + "+WebSphere:node="+ nodeName +",server=" + dtName;
			
		except:
			targetMappings  = targetMappings  + "+WebSphere:cluster="+ dtName;
			
	mapWebModuleOptionContent.append(targetMappings);
	
	print ""
	print "options : ", options
	
	AdminApp.install(installableApp, options);
	AdminConfig.save();
	print "installed application from ", installableApp
		
	print ""
#endDef


#*******************************************************************************************************
# main
#*******************************************************************************************************
numerrors = 0

print "*************************************************************************************"
print "start: installing Business Leader Apps"
print ""
print ""
print "Operating system: " + System.getProperty("os.name")
print ""
print "Command line options: " + `sys.argv`
print ""
print "installation root: " + System.getProperty("was.install.root")
print ""
print "profile root: " + System.getProperty("user.install.root")
print "profile configuration repository root: " + System.getProperty("was.repository.root")
print ""
print "local node: " + System.getProperty("local.node")
print ""
print ""
getConnectionStatus()
print ""

if (connectionStatus == "ERROR"):
    print "Error: Required connection could not be established, aborting."
    numerrors = 1
else:
	
	cellType = getCellType();
	
	if (cellType == 'STANDALONE'):
		print "Updating the stand-alone profile"
		
		if (isPublishingServerInstalled() == "true"):
			print "The Publishing Server has been installed"
			print ""	
					
			installBusinessLeaderApps();
		
		# is business space installed on this profile
		if (isBusinessSpaceInstalled() == "true"):
			print "The Business Space has been installed"
			print ""
			
			copyRegistrationFiles();
			copyTemplateFiles();					
			
	elif (cellType == 'DISTRIBUTED'):	
		print "Updating the distributed (dmgr or customized) profile"
		# check whether the local node is a deployment manager Node
		isDMgrNode = isDMgrNode();
		
		if (isDMgrNode == "True"):	
			print "Updating the distributed dmgr profile"
							
			if (isPublishingServerInstalled() == "true"):
				print "The Publishing Server has been installed"
				print ""	
								
				installBusinessLeaderApps();
			
		else:
			# Is business space installed on this profile
			if (isBusinessSpaceInstalled() == "true"):
				print "The Business Space has been installed"
				print ""
				
				copyRegistrationFiles();
				copyTemplateFiles();	
										
print ""
print "end: installing Business Leader Apps"		
print "*************************************************************************************"		